#!/bin/bash
#
# # $Id: op5restore.sh,v 1.2 2006/06/21 07:51:20 jd Exp $
#
#

# Check that user is root, if not - bail
if [ $UID -ne 0 ]; then
  echo "You must be root in order to run this script."
  exit 1
fi

# Backupfile as arg
numargs=$#
if [ $numargs -ne 1  ]; then
	echo "Wrong number of arguments"
	echo "Usage: $0 <backupfile>"
	exit 1
fi

# Check backup version
if echo $1 | grep -q "\.backup$"; then
	if tar -tf $1 | grep -q "^version$"; then
		echo "This is op5-backup v2.1 (or later) backup file. Use 'op5-restore' application to restore it."
	else
		echo "This is op5-backup v2.0 backup file. Use 'op5-restore-2.0' application to restore it."
	fi
	exit 1
fi

# Prompt to make sure user is serious...
echo "This will restore the backup and overwrite all current settings."
echo -n "Continue restoring backup? [yes/no]: "
read ans

if [ "$ans" != "yes" ]; then
	echo "Aborting, files unchanged"
	exit 1
fi


# remove any old database dumps (if user allows)
if [ "$(ls /opt/databases/*.sql 2> /dev/null)" ]; then
    echo -n "Old database dumps found in /opt/databases. Continue and delete them? [yes/no]: "
    read ans

    if [ "$ans" != "yes" ]; then
        echo "Aborting"
	exit 1
    fi
    rm -rf /opt/databases/*.sql
fi


# Unpack in /
cd /


if [ -f $1 ]; then
	echo "Restoring backupfile '$1'"
else
	echo "File '$1' not found, only absolute path is accepted"
	exit 1
fi

# Verify that all op5 products are installed
echo "Checking that all op5 products are installed"
missing_products=0
backuped_files=`tar tvzf $1`
monitor_in_backup=`echo $backuped_files |grep opt/monitor/etc`
if [ $? -eq 0 ]; then
  monitor_installed=`rpm -q monitor`
  if [ $? -eq 0 ]; then
    echo "op5 Monitor - Installed"
	monitor_installed_version=$(rpm -q --qf="%{version}" op5-monitor-release)
	monitor_version_in_backup=`echo $backuped_files |grep opt/op5-versions/op5-monitor-version.txt`
	if [ $? -eq 0 ]; then
	  monitor_backup_version=$(tar -xvf $1 opt/op5-versions/op5-monitor-version.txt | xargs cat)
	  if [ "$monitor_backup_version" == "$monitor_installed_version" ]; then
	    echo "op5 Monitor version matches"
	  else
	    echo "Warning:"
		echo " Installed op5 Monitor version = $monitor_installed_version"
		echo " Backed up op5 Monitor version = $monitor_backup_version"
		echo -n "Continue anyway [y/n]:"
		read answer
		if [[ "$answer" != "y" && "$answer" != "Y" ]]; then
		  echo "Aborting"
		  exit 1
		fi
	  fi
	fi
  else
    echo "op5 Monitor - NOT installed"
    missing_products=1
  fi
fi

statistics_in_backup=`echo $backuped_files | grep opt/statistics`
if [ $? -eq 0 ]; then
  statistics_installed=`rpm -q statistics`
  if [ $? -eq 0 ]; then
    echo "op5 Statistics - Installed"
  else
    echo "op5 Statistics - NOT installed"
    missing_products=1
  fi
fi
logserver_in_backup=`echo $backuped_files | grep opt/databases/logs`
if [ $? -eq 0 ]; then
  logserver_installed=`rpm -q op5-logserver`
  if [ $? -eq 0 ]; then
    echo "op5 Logserver - Installed"
  else
    echo "op5 Logserver - NOT installed"
    missing_products=1
  fi
fi
if [ $missing_products -ne 0 ]; then
  echo "Install missing products and try again"
  exit 1
else
  echo "All products installed correctly"
fi

echo "Restoring configuration and data"
# Unpack it all, forcing ownership / permissions to what
# they were when the backup was taken
tar -xz --preserve-permissions --same-owner -f $1 -C / --no-overwrite-dir

# Convert rrd dumps back to rrd format.
if [ -d /opt/statistics/rra ]; then
  echo -n "Restoring rrd files for op5 Statistics, this might take a while."
#  rrdfiles=$(ls /opt/statistics/rra/*.rrd.xml)
  for f in /opt/statistics/rra/*.rrd.xml.bz2; do
    bunzip2 -f $f
	f=$(echo $f | sed 's/.bz2//')
    rrdfile=$(echo $f | sed 's/.xml//')
	rrdtool restore -f $f $rrdfile
	rm -f $f
  done
  chown stats:apache /opt/statistics/rra/*.rrd
  chmod 664 /opt/statistics/rra/*.rrd
  echo " - Done."
fi
if [ -d /opt/monitor/op5/pnp/perfdata ]; then
  echo -n "Restoring rrd files for op5 Monitor, this might take a while."
  pnpdir=$(ls /opt/monitor/op5/pnp/perfdata)
  for d in $pnpdir; do
    if [ -d /opt/monitor/op5/pnp/perfdata/$d ]; then
	  for f in $(ls /opt/monitor/op5/pnp/perfdata/$d/*.rrd.xml.bz2); do
	    bunzip2 -f $f
		f=$(echo $f | sed 's/.bz2//')
	    rrdfile=$(echo $f | sed 's/.xml//')
	    rrdtool restore -f $f $rrdfile
		rm -f $f
	  done
	  chown monitor:apache /opt/monitor/op5/pnp/perfdata/$d/*.rrd
	  chmod 664 /opt/monitor/op5/pnp/perfdata/$d/*.rrd
    fi
  done
  echo " - Done."
fi
# FIXME! We do not currently handle nagiosgraph conversion... op5backup do not support it

# Restore databases
if [ -f /opt/databases/monitor_gui*.sql ]; then
	echo -n "Restoring monitor_gui database. "
	mysql monitor_gui < /opt/databases/monitor_gui*.sql
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /opt/databases/monitor_webconfig*.sql ]; then
	echo -n "Restoring monitor_webconfig database. "
	mysql monitor_webconfig < /opt/databases/monitor_webconfig*.sql
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /opt/databases/nacoma*.sql ]; then
	echo -n "Restoring nacoma database. "
	mysql nacoma < /opt/databases/nacoma*.sql
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /opt/databases/monitor_reports*.sql ]; then
	echo -n "Restoring monitor_reports database. "
	mysql monitor_reports < /opt/databases/monitor_reports*.sql
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /opt/databases/statistics*.sql ]; then
	echo -n "Restoring statistics database. "
	mysql statistics < /opt/databases/statistics*.sql
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /opt/databases/logs*.sql ]; then
	echo -n "Restoring logs database. "
	if [ $(rpm -q --queryformat="%{VERSION}\n"  `rpm -qa | grep logserver-gui` | cut -d '.' -f 1) -eq 2 ]
	then
	    mysql logs < /opt/databases/logs*.sql
	else
	    su -l op5lsu -c "psql -U op5lsu -d postgres -c 'DROP DATABASE IF EXISTS op5lsu'" && psql -U op5lsu postgres -f /opt/databases/logs*.sql && su -l op5lsu -c "psql -U op5lsu -d op5lsu -c \"UPDATE tables_registry SET location = location & ~get_location_mask('database')\""
	fi
	if [ $? -eq 0 ]; then
		echo " Successfully restored database."
	else
		echo "ERROR! Database restore failed. Database dump is found in /opt/databases/..."
	fi
fi
if [ -f /tmp/slapcat.ldif ]; then
	killall slapd
	sleep 3
	killall -9 slapd
	sleep 1
	slapadd -l /tmp/slapcat.ldif && rm -f /tmp/slapcat.ldif
	chown -R ldap /var/lib/ldap/*
	service ldap start
fi

echo ""
echo "Backup restored."
echo "Please reboot the server in order for all changes to take effect"
